@@ -31,6 +31,7 @@ from typing import Any, Dict, List, Optional, Tuple, Union
 
 import pyppmd
 import pyzstd
+from zipfile_deflate64 import deflate64  # type: ignore
 from Cryptodome.Cipher import AES
 from Cryptodome.Random import get_random_bytes
 
@@ -45,6 +46,7 @@ from py7zr.properties import (
     FILTER_COPY,
     FILTER_CRYPTO_AES256_SHA256,
     FILTER_DEFLATE,
+    FILTER_DEFLATE64,
     FILTER_DELTA,
     FILTER_IA64,
     FILTER_LZMA,
@@ -261,6 +263,26 @@ class DeflateDecompressor(ISevenZipDecompressor):
         return self._decompressor.decompress(data)
 
 
+class Deflate64Compressor(ISevenZipCompressor):
+    def __init__(self):
+        raise RuntimeError("Deflate64 compression not implemented yet")
+
+
+class Deflate64Decompressor(ISevenZipDecompressor):
+    def __init__(self):
+        self.flushed = False
+        self._decompressor = deflate64.Deflate64()
+
+    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int = -1) -> bytes:
+        if len(data) == 0:
+            if self.flushed:
+                return b""
+            else:
+                self.flushed = True
+                return self._decompressor.flush()
+        return self._decompressor.decompress(data)
+
+
 class CopyCompressor(ISevenZipCompressor):
     def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes:
         return bytes(data)
@@ -286,7 +308,7 @@ class PpmdDecompressor(ISevenZipDecompressor):
             order, mem, _, _ = struct.unpack("<BLBB", properties)
         else:
             raise UnsupportedCompressionMethodError(properties, "Unknown size of properties is passed")
-        self.decoder = pyppmd.Ppmd7Decoder(order, mem)  # type: ignore
+        self.decoder = pyppmd.Ppmd7Decoder(order, mem)
 
     def decompress(self, data: Union[bytes, bytearray, memoryview], max_length=-1) -> bytes:
         if max_length <= 0:
@@ -442,7 +464,7 @@ class BrotliCompressor(ISevenZipCompressor):
         return self._compressor.flush()
 
 
-class BrotliDecompressor:
+class BrotliDecompressor(ISevenZipDecompressor):
     def __init__(self, properties: bytes, block_size: int):
         if len(properties) != 3:
             raise UnsupportedCompressionMethodError(properties, "Unknown size of properties are passed")
@@ -453,23 +475,21 @@ class BrotliDecompressor:
                     properties[0], properties[1], brotli_major, brotli_minor
                 ),
             )
+        self._prefix_checked = False
         self._decompressor = brotli.Decompressor()
-        self.decompress = self._decompress1
 
-    def _decompress(self, data):
-        return self._decompressor.process(data)
-
-    def _decompress1(self, data):
-        # check first 4bytes
-        if data[:4] == b"\x50\x2a\x4d\x18":
-            raise UnsupportedCompressionMethodError(
-                data[:4], "Unauthorized and modified Brotli data (skipable frame) found."
-            )
-        self.decompress = self._decompress
+    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int = -1):
+        if not self._prefix_checked:
+            # check first 4bytes
+            if data[:4] == b"\x50\x2a\x4d\x18":
+                raise UnsupportedCompressionMethodError(
+                    data[:4], "Unauthorized and modified Brotli data (skipable frame) found."
+                )
+            self._prefix_checked = True
         return self._decompressor.process(data)
 
 
-class ZstdCompressor:
+class ZstdCompressor(ISevenZipCompressor):
     def __init__(self, level: int):
         self.compressor = pyzstd.ZstdCompressor(level)
 
@@ -480,13 +500,15 @@ class ZstdCompressor:
         return self.compressor.flush()
 
 
-class ZstdDecompressor:
+class ZstdDecompressor(ISevenZipDecompressor):
     def __init__(self, properties: bytes, blocksize: int):
-        if len(properties) not in [3, 5] or (properties[0], properties[1], 0) > pyzstd.zstd_version_info:
-            raise UnsupportedCompressionMethodError
+        if len(properties) not in [3, 5]:
+            raise UnsupportedCompressionMethodError(properties, "Zstd takes 3 or 5 bytes properties.")
+        if (properties[0], properties[1], 0) > pyzstd.zstd_version_info:
+            raise UnsupportedCompressionMethodError(properties, "Zstd version of archive is higher than us.")
         self.decompressor = pyzstd.ZstdDecompressor()
 
-    def decompress(self, data: Union[bytes, bytearray, memoryview]) -> bytes:
+    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int = -1) -> bytes:
         return self.decompressor.decompress(data)
 
 
@@ -497,6 +519,7 @@ algorithm_class_map = {
     FILTER_BZIP2: (bz2.BZ2Compressor, bz2.BZ2Decompressor),
     FILTER_COPY: (CopyCompressor, CopyDecompressor),
     FILTER_DEFLATE: (DeflateCompressor, DeflateDecompressor),
+    FILTER_DEFLATE64: (Deflate64Compressor, Deflate64Decompressor),
     FILTER_CRYPTO_AES256_SHA256: (AESCompressor, AESDecompressor),
     FILTER_X86: (BCJEncoder, BCJDecoder),
     FILTER_ARM: (BcjArmEncoder, BcjArmDecoder),
@@ -511,7 +534,7 @@ class LZMA1Decompressor(ISevenZipDecompressor):
         self._decompressor = lzma.LZMADecompressor(format=lzma.FORMAT_RAW, filters=filters)
         self.unpacksize = unpacksize
 
-    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int) -> bytes:
+    def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int = -1) -> bytes:
         return self._decompressor.decompress(data, max_length)
 
 
@@ -538,11 +561,13 @@ class SevenZipDecompressor:
         else:
             self.block_size = get_default_blocksize()
         if len(coders) > 4:
-            raise UnsupportedCompressionMethodError("Maximum cascade of filters is 4 but got {}.".format(len(coders)))
+            raise UnsupportedCompressionMethodError(
+                coders, "Maximum cascade of filters is 4 but got {}.".format(len(coders))
+            )
         self.methods_map = [SupportedMethods.is_native_coder(coder) for coder in coders]  # type: List[bool]
         # Check if password given for encrypted archive
         if SupportedMethods.needs_password(coders) and password is None:
-            raise PasswordRequired("Password is required for extracting given archive.")
+            raise PasswordRequired(coders, "Password is required for extracting given archive.")
         # Check filters combination and required parameters
         if len(coders) >= 2:
             target_compressor = False
@@ -600,7 +625,7 @@ class SevenZipDecompressor:
                     else:
                         self.chain.append(self._get_alternative_decompressor(coders[i], unpacksizes[i], password))
         else:
-            raise UnsupportedCompressionMethodError
+            raise UnsupportedCompressionMethodError(coders, "Combination order of methods is not supported.")
 
     def _decompress(self, data, max_length: int):
         for i, decompressor in enumerate(self.chain):
@@ -616,25 +641,38 @@ class SevenZipDecompressor:
                 raise EOFError
         return data
 
-    def decompress(self, fp, max_length: int = -1) -> bytes:
+    def _read_data(self, fp):
         # read data from disk
+        # determine read siize
+        #    rest_size: rest size of packed data
+        #    unused_s: unused packed data size
+        #  size to consume for target file is smaller one from
+        #    rest_size - unused_s
+        #    block_size - unused_s
         rest_size = self.input_size - self.consumed
-        read_size = min(rest_size, self.block_size)
-        data = fp.read(read_size)
-        self.consumed += len(data)
-        #
+        unused_s = len(self._unused)
+        read_size = min(rest_size - unused_s, self.block_size - unused_s)
+        if read_size > 0:
+            data = fp.read(read_size)
+            self.consumed += len(data)
+        else:
+            data = b""
+        return data
+
+    def decompress(self, fp, max_length: int = -1) -> bytes:
         if max_length < 0:
+            data = self._read_data(fp)
             res = self._buf[self._pos :] + self._decompress(self._unused + data, max_length)
             self._buf = bytearray()
             self._unused = bytearray()
             self._pos = 0
         else:
             current_buf_len = len(self._buf) - self._pos
-            if current_buf_len >= max_length:
-                self._unused.extend(data)
+            if current_buf_len >= max_length:  # we already have enough data
                 res = self._buf[self._pos : self._pos + max_length]
                 self._pos += max_length
             else:
+                data = self._read_data(fp)
                 if len(self._unused) > 0:
                     tmp = self._decompress(self._unused + data, max_length)
                     self._unused = bytearray()
@@ -663,9 +701,9 @@ class SevenZipDecompressor:
         lzma1 = False
         for coder in coders:
             if coder["numinstreams"] != 1 or coder["numoutstreams"] != 1:
-                raise UnsupportedCompressionMethodError("Only a simple compression method is currently supported.")
+                raise UnsupportedCompressionMethodError(coders, "Only a simple compression method is currently supported.")
             if not SupportedMethods.is_native_coder(coder):
-                raise UnsupportedCompressionMethodError
+                raise UnsupportedCompressionMethodError(coders, "Non python native method is requested.")
             properties = coder.get("properties", None)
             filter_id = SupportedMethods.get_filter_id(coder)
             if filter_id == FILTER_LZMA:
@@ -694,12 +732,12 @@ class SevenZipDecompressor:
             return algorithm_class_map[filter_id][1](size=unpacksize)
         # Check supported?
         if SupportedMethods.is_native_coder(coder):
-            raise UnsupportedCompressionMethodError("Unknown method code:{}".format(coder["method"]))
+            raise UnsupportedCompressionMethodError(coder, "Unknown method code:{}".format(coder["method"]))
         if filter_id not in algorithm_class_map:
-            raise UnsupportedCompressionMethodError("Unknown method filter_id:{}".format(filter_id))
+            raise UnsupportedCompressionMethodError(coder, "Unknown method filter_id:{}".format(filter_id))
         if algorithm_class_map[filter_id][1] is None:
             raise UnsupportedCompressionMethodError(
-                "Decompression is not supported by {}.".format(SupportedMethods.get_method_name_id(filter_id))
+                coder, "Decompression is not supported by {}.".format(SupportedMethods.get_method_name_id(filter_id))
             )
         #
         if SupportedMethods.is_crypto_id(filter_id):
@@ -740,16 +778,31 @@ class SevenZipCompressor:
         else:
             self.filters = filters
         if len(self.filters) > 4:
-            raise UnsupportedCompressionMethodError("Maximum cascade of filters is 4 but got {}.".format(len(self.filters)))
+            raise UnsupportedCompressionMethodError(
+                filters, "Maximum cascade of filters is 4 but got {}.".format(len(self.filters))
+            )
         self.methods_map = [SupportedMethods.is_native_filter(filter) for filter in self.filters]
         self.coders: List[Dict[str, Any]] = []
         if all(self.methods_map) and SupportedMethods.is_compressor(self.filters[-1]):  # all native
             self._set_native_compressors_coders(self.filters)
             return
         #
-        for i, f in enumerate(self.filters):
-            if f["id"] == FILTER_X86:
-                self.methods_map[i] = False
+        has_lzma2 = False
+        for f in self.filters:
+            if f["id"] == FILTER_LZMA2:
+                has_lzma2 = True
+                break
+        if not has_lzma2:
+            # when specified other than lzma2, BCJ filters should be alternative
+            for i, f in enumerate(self.filters):
+                if (
+                    f["id"] == FILTER_X86
+                    or f["id"] == FILTER_ARM
+                    or f["id"] == FILTER_ARMTHUMB
+                    or f["id"] == FILTER_SPARC
+                    or f["id"] == FILTER_POWERPC
+                ):
+                    self.methods_map[i] = False
         #
         if not any(self.methods_map):  # all alternative
             for f in filters:
@@ -758,7 +811,7 @@ class SevenZipCompressor:
             self._set_native_compressors_coders(self.filters[:-1])
             self._set_alternate_compressors_coders(self.filters[-1], password)
         else:
-            raise UnsupportedCompressionMethodError
+            raise UnsupportedCompressionMethodError(filters, "Unknown combination of methods.")
 
     def _set_native_compressors_coders(self, filters):
         self.chain.append(lzma.LZMACompressor(format=lzma.FORMAT_RAW, filters=filters))
@@ -770,7 +823,7 @@ class SevenZipCompressor:
         filter_id = alt_filter["id"]
         properties = None
         if filter_id not in algorithm_class_map:
-            raise UnsupportedCompressionMethodError
+            raise UnsupportedCompressionMethodError(filter_id, "Unknown filter_id is given.")
         elif SupportedMethods.is_crypto_id(filter_id):
             compressor = algorithm_class_map[filter_id][0](password)
         elif SupportedMethods.need_property(filter_id):
@@ -975,6 +1028,14 @@ class SupportedMethods:
             "filter_id": FILTER_BROTLI,
             "type": MethodsType.compressor,
         },
+        {
+            "id": COMPRESSION_METHOD.MISC_DEFLATE64,
+            "name": "DEFLATE64",
+            "native": False,
+            "need_prop": False,
+            "filter_id": FILTER_DEFLATE64,
+            "type": MethodsType.compressor,
+        },
         {
             "id": COMPRESSION_METHOD.CRYPT_AES256_SHA256,
             "name": "7zAES",
@@ -1005,7 +1066,7 @@ class SupportedMethods:
     def is_native_filter(cls, filter) -> bool:
         method = cls._find_method("filter_id", filter["id"])
         if method is None:
-            raise UnsupportedCompressionMethodError
+            raise UnsupportedCompressionMethodError(filter["id"], "Unknown method id is given.")
         return method["native"]
 
     @classmethod
@@ -1022,35 +1083,35 @@ class SupportedMethods:
     def is_native_coder(cls, coder) -> bool:
         method = cls._find_method("id", coder["method"])
         if method is None:
-            raise UnsupportedCompressionMethodError(coder["method"], "Found an unknown id of method.")
+            cls.raise_unsupported_method_id(coder)
         return method["native"]
 
     @classmethod
     def need_property(cls, filter_id):
         method = cls._find_method("filter_id", filter_id)
         if method is None:
-            raise UnsupportedCompressionMethodError(filter_id, "Found an unknown filter id.")
+            raise UnsupportedCompressionMethodError(filter_id, "Found an sunpported filter id.")
         return method["need_prop"]
 
     @classmethod
     def is_crypto_id(cls, filter_id) -> bool:
         method = cls._find_method("filter_id", filter_id)
         if method is None:
-            raise UnsupportedCompressionMethodError(filter_id, "Found an unknown filter id.")
+            cls.raise_unsupported_filter_id(filter_id)
         return method["type"] == MethodsType.crypto
 
     @classmethod
     def get_method_id(cls, filter_id) -> bytes:
         method = cls._find_method("filter_id", filter_id)
         if method is None:
-            raise UnsupportedCompressionMethodError(filter_id, "Found an unknown filter id.")
+            cls.raise_unsupported_filter_id(filter_id)
         return method["id"]
 
     @classmethod
     def get_coder(cls, filter) -> Dict[str, Any]:
         method = cls.get_method_id(filter["id"])
         if filter["id"] in [lzma.FILTER_LZMA1, lzma.FILTER_LZMA2, lzma.FILTER_DELTA]:
-            properties = lzma._encode_filter_properties(filter)  # type: Optional[bytes] # type: ignore  # noqa
+            properties: Optional[bytes] = lzma._encode_filter_properties(filter)  # type: ignore  # noqa
         else:
             properties = None
         return {
@@ -1070,6 +1131,36 @@ class SupportedMethods:
                 return True
         return False
 
+    @classmethod
+    def raise_unsupported_method_id(cls, coder):
+        if coder["method"] == COMPRESSION_METHOD.P7Z_BCJ2:
+            raise UnsupportedCompressionMethodError(
+                coder["method"],
+                "BCJ2 filter is not supported by py7zr."
+                " Please consider to contribute to XZ/liblzma project"
+                " and help Python core team implementing it."
+                " Or please use another tool to extract it.",
+            )
+        if coder["method"] == COMPRESSION_METHOD.MISC_DEFLATE64:
+            raise UnsupportedCompressionMethodError(
+                coder["method"],
+                "DEFLATE64 compression is not supported by py7zr yet."
+                " Please check the progress in py7zr project home page.",
+            )
+        if coder["method"] == COMPRESSION_METHOD.MISC_LZ4:
+            raise UnsupportedCompressionMethodError(
+                coder["method"], "Archive is compressed by an unsupported algorythm LZ4."
+            )
+        raise UnsupportedCompressionMethodError(
+            coder["method"], "Archive is compressed by an unsupported compression algorythm."
+        )
+
+    @classmethod
+    def raise_unsupported_filter_id(cls, filter_id):
+        raise UnsupportedCompressionMethodError(
+            filter_id, "Found an unsupported filter id is specified." "Please use another compression method."
+        )
+
 
 def get_methods_names(coders_lists: List[List[dict]]) -> List[str]:
     # list of known method names with a display priority order
@@ -1079,7 +1170,7 @@ def get_methods_names(coders_lists: List[List[dict]]) -> List[str]:
         "LZMA",
         "BZip2",
         "DEFLATE",
-        "DEFLATE64*",
+        "DEFLATE64",
         "delta",
         "COPY",
         "PPMd",
@@ -1097,7 +1188,6 @@ def get_methods_names(coders_lists: List[List[dict]]) -> List[str]:
     unsupported_methods = {
         COMPRESSION_METHOD.P7Z_BCJ2: "BCJ2*",
         COMPRESSION_METHOD.MISC_LZ4: "LZ4*",
-        COMPRESSION_METHOD.MISC_DEFLATE64: "DEFLATE64*",
     }
     methods_names = []
     for coders in coders_lists:
